@@ -15,7 +15,11 @@ import libsbml
 from typing_extensions import Literal  # 3.7 compatability
 
 from .basemodel import BaseModel
-from .utils import convert_id_to_sbml, warning_on_one_line
+from .utils import (
+    convert_id_to_sbml,
+    get_formatted_function_source_code,
+    warning_on_one_line,
+)
 
 warnings.formatwarning = warning_on_one_line  # type: ignore
 
@@ -250,8 +254,7 @@ class ParameterMixin(BaseModel):
     ##########################################################################
     # Source code functions
     ##########################################################################
-
-    def _generate_parameters_source_code(self, *, include_meta_info: bool = True) -> str:
+    def _generate_constant_parameters_source_code(self, *, include_meta_info: bool = True) -> str:
         """Generate modelbase source code for parameters.
 
         This is mainly used for the generate_model_source_code function.
@@ -266,13 +269,56 @@ class ParameterMixin(BaseModel):
         parameter_modelbase_code : str
             Source code generating the modelbase parameters
         """
-        parameters = repr(self.parameters)
+        parameters = repr({k: v for k, v in self.parameters.items() if k not in self.derived_parameters})
+        # derived_pars =
         if include_meta_info:
             meta_info = self._get_nonzero_meta_info(component="parameters")
             if bool(meta_info):
                 return f"m.add_parameters(parameters={parameters}, meta_info={meta_info})"
         return f"m.add_parameters(parameters={parameters})"
 
+    def _generate_derived_parameters_source_code(self) -> tuple[str, str]:
+        """Generate modelbase source code for parameters.
+
+        This is mainly used for the generate_model_source_code function.
+
+        Parameters
+        ----------
+        include_meta_info : bool
+            Whether to include the parameter meta info
+
+        Returns
+        -------
+        parameter_modelbase_code : str
+            Source code generating the modelbase parameters
+        """
+        fns: set[str] = set()
+        pars: list[str] = []
+        for name, module in self.derived_parameters.items():
+            function = module["function"]
+            parameters = module["parameters"]
+
+            function_code = get_formatted_function_source_code(
+                function_name=name, function=function, function_type="module"
+            )
+            fns.add(function_code)
+            pars.append(
+                (
+                    "m.add_derived_parameter(\n"
+                    f"    parameter_name={repr(name)},\n"
+                    f"    function={function.__name__},\n"
+                    f"    parameters={parameters},\n"
+                    ")"
+                )
+            )
+        return "\n".join(sorted(fns)), "\n".join(pars)
+
+    def _generate_parameters_source_code(self, *, include_meta_info: bool = True) -> tuple[str, str, str]:
+        return (
+            self._generate_constant_parameters_source_code(include_meta_info=include_meta_info),
+            *self._generate_derived_parameters_source_code(),
+        )
+
     ##########################################################################
     # SBML functions
     ##########################################################################
