@@ -6,17 +6,40 @@ import copy
 import itertools as it
 import warnings
 from collections import defaultdict
-from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Union,
+    cast,
+)
 
 import libsbml
 import numpy as np
 import pandas as pd
 
+from ...core import (
+    AlgebraicMixin,
+    BaseModel,
+    CompoundMixin,
+    ParameterMixin,
+    RateMixin,
+    StoichiometricMixin,
+)
 from ...core.compoundmixin import Compound
-from ...core.ratemixin import RateMeta
+from ...core.ratemixin import Rate, RateMeta
 from ...core.utils import convert_id_to_sbml, warning_on_one_line
 from ...typing import Array, ArrayLike
-from .model import Model
+from . import _AbstractRateModel
+
+if TYPE_CHECKING:
+    from . import Model
 
 warnings.formatwarning = warning_on_one_line  # type: ignore
 
@@ -30,7 +53,7 @@ def total_concentration(*args: float) -> List[Array]:
     return [np.sum(args, axis=0)]  # type: ignore
 
 
-class LabelModel(Model):
+class LabelModel(_AbstractRateModel, BaseModel):
     """LabelModel."""
 
     def __init__(
@@ -38,21 +61,21 @@ class LabelModel(Model):
         parameters: dict[str, float] | None = None,
         compounds: List[str] | None = None,
         algebraic_modules: dict[str, dict[str, Any]] | None = None,
-        rate_stoichiometries: Dict[str, Dict[str, int]] | None = None,
-        rates: dict[str, dict[str, Any]] | None = None,
+        rate_stoichiometries: Dict[str, Dict[str, float]] | None = None,
+        rates: dict[str, Rate] | None = None,
+        functions: dict | None = None,
         meta_info: dict | None = None,
     ):
+        BaseModel.__init__(self, meta_info=meta_info)
+        CompoundMixin.__init__(self, compounds=compounds)
+        ParameterMixin.__init__(self, parameters=parameters)
+        AlgebraicMixin.__init__(self, algebraic_modules=algebraic_modules)
+        StoichiometricMixin.__init__(self, rate_stoichiometries=rate_stoichiometries)
+        RateMixin.__init__(self, rates=rates, functions=functions)
+        self.meta_info["model"].sbo = "SBO:0000062"  # continuous framework
         self.label_compounds: Dict[str, Dict[str, Any]] = {}
         self.nonlabel_compounds: List[str] = []
         self.base_reactions: Dict[str, Dict[str, Any]] = {}
-        super().__init__(
-            parameters=parameters,
-            compounds=compounds,
-            algebraic_modules=algebraic_modules,
-            rate_stoichiometries=rate_stoichiometries,
-            rates=rates,
-            meta_info=meta_info,
-        )
 
     def __enter__(self) -> LabelModel:
         """Enter the context manager.
@@ -149,7 +172,7 @@ class LabelModel(Model):
         # Create moiety for total compound concentration
         self.add_algebraic_module(
             module_name=base_compound + "__total",
-            function=total_concentration,
+            function=total_concentration,  # type: ignore
             compounds=label_names,
             derived_compounds=[base_compound + "__total"],
             modifiers=None,
@@ -382,16 +405,20 @@ class LabelModel(Model):
                 new_compounds.append(compound)
         return new_compounds
 
-    def add_algebraic_module(  # type: ignore
+    def add_algebraic_module(
         self,
         module_name: str,
-        function: Callable,
+        function: Union[Callable[..., float], Callable[..., Iterable[float]]],
         compounds: List[str] | None = None,
         derived_compounds: List[str] | None = None,
         modifiers: List[str] | None = None,
         parameters: List[str] | None = None,
+        dynamic_variables: List[str] | None = None,
         args: List[str] | None = None,
-    ):
+        check_consistency: bool = True,
+        sort_modules: bool = True,
+        **meta_info: Dict[str, Any],
+    ) -> None:
         """Add an algebraic module to the model.
 
         CAUTION: The Python function of the module has to return an iterable.
@@ -443,6 +470,10 @@ class LabelModel(Model):
             compounds = [i + "__total" if i in self.label_compounds else i for i in compounds]
         if modifiers is not None:
             modifiers = [i + "__total" if i in self.label_compounds else i for i in modifiers]
+        if dynamic_variables is not None:
+            dynamic_variables = [i + "__total" if i in self.label_compounds else i for i in dynamic_variables]
+        if args is not None:
+            args = [i + "__total" if i in self.label_compounds else i for i in args]
         super().add_algebraic_module(
             module_name=module_name,
             function=function,
@@ -450,7 +481,11 @@ class LabelModel(Model):
             derived_compounds=derived_compounds,
             modifiers=modifiers,
             parameters=parameters,
+            dynamic_variables=dynamic_variables,
             args=args,
+            check_consistency=check_consistency,
+            sort_modules=sort_modules,
+            **meta_info,
         )
 
     def _get_external_labels(
@@ -473,6 +508,63 @@ class LabelModel(Model):
             return "".join(external_label_string)
         return ""
 
+    def add_reaction(
+        self,
+        rate_name: str,
+        function: Callable[..., float],
+        stoichiometry: Dict[str, float],
+        modifiers: List[str] | None = None,
+        parameters: List[str] | None = None,
+        dynamic_variables: List[str] | None = None,
+        args: List[str] | None = None,
+        reversible: bool = False,
+        check_consistency: bool = True,
+        **meta_info: Dict[str, Any],
+    ) -> None:
+        """Add a reaction to the model.
+
+        Shortcut for add_rate and add stoichiometry functions.
+
+        See Also
+        --------
+        add_rate
+        add_stoichiometry
+
+        Examples
+        --------
+        >>> add_reaction(
+        >>>     rate_name="v1",
+        >>>     function=mass_action,
+        >>>     stoichiometry={"X": -1, "Y": 1},
+        >>>     parameters=["k2"],
+        >>> )
+
+        >>> add_reaction(
+        >>>     rate_name="v1",
+        >>>     function=reversible_mass_action,
+        >>>     stoichiometry={"X": -1, "Y": 1},
+        >>>     parameters=["k1_fwd", "k1_bwd"],
+        >>>     reversible=True,
+        >>> )
+        """
+        substrates = [k for k, v in stoichiometry.items() if v < 0]
+        products = [k for k, v in stoichiometry.items() if v > 0]
+
+        self.add_rate(
+            rate_name=rate_name,
+            function=function,
+            substrates=substrates,
+            products=products,
+            dynamic_variables=dynamic_variables,
+            modifiers=modifiers,
+            parameters=parameters,
+            reversible=reversible,
+            args=args,
+            check_consistency=check_consistency,
+            **meta_info,
+        )
+        self.add_stoichiometry(rate_name=rate_name, stoichiometry=stoichiometry)
+
     def _add_base_reaction(
         self,
         *,
@@ -748,6 +840,16 @@ class LabelModel(Model):
             # args=args,
         )
 
+    def remove_reaction(self, rate_name: str) -> None:
+        """Remove a reaction from the model.
+
+        Parameters
+        ----------
+        rate_name : str
+        """
+        self.remove_rate(rate_name=rate_name)
+        self.remove_rate_stoichiometry(rate_name=rate_name)
+
     def remove_labelmap_reaction(self, rate_name: str) -> None:
         """Remove all variants of a base reaction.
 
@@ -812,6 +914,96 @@ class LabelModel(Model):
                 y0[base_compound + suffix] = concentration
         return y0
 
+    def get_full_concentration_dict(
+        self,
+        y: Union[Dict[str, float], Dict[str, Array], ArrayLike, Array],
+        t: Union[float, ArrayLike, Array] = 0.0,
+    ) -> Dict[str, Array]:
+        """Calculate the derived variables (at time(s) t).
+
+        Examples
+        --------
+        >>> get_full_concentration_dict(y=[0, 0])
+        >>> get_full_concentration_dict(y={"X": 0, "Y": 0})
+        """
+        self._update_derived_parameters()
+        if isinstance(t, (int, float)):
+            t = [t]
+        t = np.array(t)
+        if isinstance(y, dict):
+            y = {k: np.ones(len(t)) * v for k, v in y.items()}
+        else:
+            y = dict(zip(self.get_compounds(), (np.ones((len(t), 1)) * y).T))
+        return {k: np.ones(len(t)) * v for k, v in self._get_fcd(t=t, y=y).items()}  # type: ignore
+
+    def get_fluxes_dict(
+        self,
+        y: Union[
+            Dict[str, float],
+            Dict[str, ArrayLike],
+            Dict[str, Array],
+            Array,
+            ArrayLike,
+        ],
+        t: Union[float, ArrayLike, Array] = 0.0,
+    ) -> Dict[str, Array]:
+        """Calculate the fluxes at time point(s) t."""
+        self._update_derived_parameters()
+        fcd = self.get_full_concentration_dict(y=y, t=t)  # type: ignore
+        ones = np.ones(len(fcd["time"]))
+        if len(fcd["time"]) == 1:
+            return {k: ones * v for k, v in self._get_fluxes(fcd=fcd).items()}  # type: ignore
+        return {k: ones * v for k, v in self._get_fluxes_array(fcd=fcd).items()}  # type: ignore
+
+    def get_fluxes_array(
+        self,
+        y: Union[
+            Dict[str, float],
+            Dict[str, ArrayLike],
+            Dict[str, Array],
+            Array,
+            ArrayLike,
+        ],
+        t: Union[float, ArrayLike, Array] = 0.0,
+    ) -> Array:
+        """Calculate the fluxes at time point(s) t."""
+        return np.array(list(self.get_fluxes_dict(y=y, t=t).values())).T
+
+    def get_fluxes_df(
+        self,
+        y: Union[
+            Dict[str, float],
+            Dict[str, ArrayLike],
+            Dict[str, Array],
+            Array,
+            ArrayLike,
+        ],
+        t: Union[float, ArrayLike, Array] = 0.0,
+    ) -> pd.DataFrame:
+        """Calculate the fluxes at time point(s) t."""
+        if isinstance(t, (int, float)):
+            t = [t]  # type: ignore
+        return pd.DataFrame(data=self.get_fluxes_dict(y=y, t=t), index=t, columns=self.get_rate_names())
+
+    def get_right_hand_side(
+        self,
+        y: Union[
+            Dict[str, float],
+            Dict[str, ArrayLike],
+            Dict[str, Array],
+            Array,
+            ArrayLike,
+        ],
+        t: Union[float, ArrayLike, Array] = 0.0,
+    ) -> Dict[str, float]:
+        """Calculate the right hand side of the ODE system."""
+        self._update_derived_parameters()
+        fcd = self.get_full_concentration_dict(y=y, t=t)  # type: ignore
+        fcd_array = [fcd[i] for i in self.get_compounds()]
+        rhs = self._get_rhs(t=t, y=fcd_array)
+        eqs = [f"d{cpd}dt" for cpd in self.get_compounds()]
+        return dict(zip(eqs, rhs))
+
     def get_total_fluxes(
         self,
         rate_base_name: str,
@@ -925,6 +1117,24 @@ class LabelModel(Model):
             loop_count += 1
         return result
 
+    def _get_rhs(self, t: Union[float, ArrayLike, Array], y: List[Array]) -> Array:
+        """Calculate the right hand side of the ODE system.
+
+        This is the more performant version of get_right_hand_side()
+        and thus returns only an array instead of a dictionary.
+
+        Watch out that this function swaps t and y!
+        """
+        y = dict(zip(self.get_compounds(), np.array(y).reshape(-1, 1)))  # type: ignore
+        fcd = self._get_fcd(t=t, y=y)  # type: ignore
+        fluxes = self._get_fluxes(fcd=fcd)  # type: ignore
+        compounds_local = self.get_compounds()
+        dxdt = dict(zip(compounds_local, it.repeat(0.0)))
+        for k, stoc in self.stoichiometries_by_compounds.items():
+            for flux, n in stoc.items():
+                dxdt[k] += n * fluxes[flux]
+        return np.array([dxdt[i] for i in compounds_local], dtype="float")
+
     ##########################################################################
     # Model conversion functions
     ##########################################################################
@@ -936,6 +1146,8 @@ class LabelModel(Model):
 
     def to_model(self) -> Model:
         """Convert LabelModel to Model."""
+        from . import Model
+
         m = Model()
         m.add_parameters(self.parameters)
         m.add_compounds(list(self.label_compounds.keys()))
@@ -1076,3 +1288,15 @@ class LabelModel(Model):
                 # if function is not None:
                 #     kinetic_law = rxn.createKineticLaw()
                 #     kinetic_law.setMath(libsbml.parseL3Formula(function))
+
+    def _model_to_sbml(self) -> libsbml.SBMLDocument:
+        """Export model to sbml."""
+        doc = self._create_sbml_document()
+        sbml_model = self._create_sbml_model(doc=doc)
+        self._create_sbml_units(sbml_model=sbml_model)
+        self._create_sbml_compartments(sbml_model=sbml_model)
+        self._create_sbml_compounds(sbml_model=sbml_model)
+        if bool(self.algebraic_modules):
+            self._create_sbml_algebraic_modules(_sbml_model=sbml_model)
+        self._create_sbml_reactions(sbml_model=sbml_model)
+        return doc
