@@ -5,6 +5,7 @@ __all__ = [
     "LinearLabelModel",
 ]
 
+import copy
 import warnings
 from typing import Any, Dict, Iterable, List, Tuple, Union, cast
 
@@ -74,6 +75,26 @@ class LinearLabelModel(_AbstractStoichiometricModel):
             for k, v in rates.items():
                 self.add_rate(rate_name=k, **v)
 
+    def __enter__(self) -> LinearLabelModel:
+        """Enter the context manager.
+
+        Returns
+        -------
+            Deepcopy of the model object
+        """
+        self._copy = self.copy()
+        return self.copy()
+
+    def copy(self) -> LinearLabelModel:
+        """Create a deepcopy of the model.
+
+        Returns
+        -------
+        model
+            Deepcopy of the model object
+        """
+        return copy.deepcopy(self)  # type: ignore
+
     @staticmethod
     def _generate_isotope_labels(*, base_name: str, num_labels: int) -> List[str]:
         """Returns a list of all label isotopomers of the compound."""
