@@ -26,7 +26,7 @@ from ...core import (
 )
 from ...core.utils import convert_id_to_sbml
 from ...typing import Array, ArrayLike
-from . import _AbstractRateModel
+from . import LabelModel, _AbstractRateModel
 
 
 class Model(_AbstractRateModel, BaseModel):
@@ -557,7 +557,7 @@ class Model(_AbstractRateModel, BaseModel):
     # Model conversion functions
     ##########################################################################
 
-    def to_labelmodel(self, labelcompounds: Dict[str, int], labelmaps: Dict[str, List[int]]) -> Any:
+    def to_labelmodel(self, labelcompounds: Dict[str, int], labelmaps: Dict[str, List[int]]) -> LabelModel:
         """Create a LabelModel from this model.
 
         Examples
